/* faux1 terminal Apple 1 emulator
 * Portions Copyright (C) 2012 James M. Reneau (renejm@users.sourceforge.net)
 *
 * napple1 ncurses Apple 1 emulator
 * Copyright (C) 2008 Nobu Hatano
 *
 * Pom1 Apple 1 Emulator
 * Copyright (C) 2000 Verhille Arnaud
 * Copyright (C) 2006 John D. Corrado
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

// command line options with help from http://www.kernel.org/doc/man-pages/online/pages/man3/getopt.3.html

// COMMAND LINE OPTIONS:
// -a xxxx - hex address for the next binary file load default 0x300
// -b filename - load binary file at last address argument (default 0x300)
// -c command string to execute once faux1 is running
// -h - help
// -l filename - load ascii file into memory (see specifications for ascii file)
// -m - DO NOT LOAD monitor 0xFF00-0xFFFF rom file
// -n - DO NOT LOAD BASIC 0xE000-0xEFFF rom file

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#include "strings.h"
#include "m6502.h"
#include "memory.h"
#include "keyboard.h"
#include "screen.h"

int main( int argc, char *argv[] )
{

	int opt;
	int needMONITOR = 1;
	int needBASIC = 1;
	unsigned int loadAddress = 0x300;
	
	terminalSetupMode();

	resetMemory();
	resetScreen();

	/* initialize apple1 screen */
	setSpeed(1000000, 50); /* 1M Hz. Sync emulation every 50 msec */

	while ((opt = getopt(argc, argv, "a:b:c:hl:mn")) != -1) {
		switch (opt) {
			case 'a':
				if (sscanf(optarg, "%4X", &loadAddress) < 1) {
					fprintf(stderr, MEMORY_ADRESSSTARTERROR);
					return 0;
				}
				break;
			case 'b':
				loadMemoryBinary(optarg, loadAddress);
				break;
			case 'c':
				keyboardBufferString(optarg);
				keyboardBufferChar('\r');
				break;
			case 'h':
				fprintf(stderr, MAIN_COMMANDLINE_HELP, VERSION, argv[0]);
				return 0;
				break;
			case 'l':
				loadMemoryASCII(optarg);
				break;
			case 'm':
				needMONITOR = 0;
				break;
			case 'n':
				needBASIC = 0;
				break;
			default: /* '?' */
				fprintf(stderr, MAIN_COMMANDLINE_ERROR , argv[0]);
				return 0;
		}
	}

	if (needMONITOR) {
		if (!loadMonitor()) return 0;
	}
	if (needBASIC) {
		if (!loadBasic()) return 0;
	}

	/* start processor */
	resetM6502();
	startM6502();

	terminalRawMode();
	while (handleInput()) {
		runM6502();
	}
	terminalNormalMode();
	stopM6502();

	/* ending */
	return 0;
}
